package game.objects 
{
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxObject;
	import org.flixel.FlxSprite;
	import org.flixel.FlxU;
	/**
	 * ...
	 * @author Flashkick
	 */
	public class Enemy extends FlxSprite
	{
		[Embed(source = "../../../media/images/Enemy.png")] private var enemyImage:Class;
		[Embed(source = "../../../media/sound/EnemyShoot.mp3")] private var shootSound:Class;
		
		private var bullets:FlxGroup;
		private var playerBullets:FlxGroup;
		private var player:Player;
		
		private var startingY:int;
		private var rangeY:int;
		private var movingDown:Boolean;
		
		private var bulletTimer:Number;
		private var bulletCooldown:Number;
		
		private var isAlive:Boolean;
		
		public function Enemy(x:int, y:int, bullets:FlxGroup, playerBullets:FlxGroup, player:Player) 
		{
			super(x, y);
			//loadGraphic(enemyImage, false, false, 60, 60);
			loadRotatedGraphic(enemyImage, 36, -1, true);
			
			this.bullets = bullets;
			this.playerBullets = playerBullets;
			this.player = player;
			
			startingY = y;
			rangeY = 50;
			
			var rand:Number = FlxU.random();
			movingDown = rand < 0.5 ? true : false;
			
			velocity.x = 100;
			maxVelocity.y = 300;
			
			bulletTimer = 0;
			bulletCooldown = 1;
			
			isAlive = true;
		}
		
		private function onPlayerBulletOverlap(self:FlxObject, bullet:FlxObject):void {
			if (!(bullet as PlayerBullet).isAttached && isAlive) {
				flicker(30);
				isAlive = false;
				angularVelocity = 500;
				velocity.x = 250;
				Statistics.kills++;
			}
		}
		
		private function onPlayerOverlap(self:FlxObject, player:FlxObject):void {
			if (!isAlive) {
				this.player.onEnemyOverlap(this);
				kill();
			}
		}
		
		private function checkCollision():void {
			FlxU.overlap(this, player, onPlayerOverlap);
			FlxU.overlap(this, playerBullets, onPlayerBulletOverlap);
		}
		
		override public function update():void 
		{
			if (isAlive) {
				y = startingY + Math.sin(x / 45) * rangeY;
				
				bulletTimer += FlxG.elapsed;
				if (bulletTimer >= bulletCooldown) {
					bulletTimer = 0;
					var bullet:Bullet = new Bullet(x, y, player);
					bullet.velocity.x = 200;
					bullets.add(bullet);
					FlxG.play(shootSound, 0.3);
				}
			}
			
			if (x > FlxG.stage.stageWidth) {
				kill();
			}
			
			checkCollision();
			
			super.update();
		}
	}

}