package game.objects 
{
	import end.EndState;
	import menu.MenuState;
	import org.flixel.FlxEmitter;
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxObject;
	import org.flixel.FlxSprite;
	import org.flixel.FlxU;
	/**
	 * ...
	 * @author Flashkick
	 */
	public class Player extends FlxSprite
	{
		[Embed(source = "../../../media/images/Player.png")] private var playerImage:Class;
		[Embed(source = "../../../media/sound/PowerUp.mp3")] private var powerUpSound:Class;
		[Embed(source = "../../../media/sound/PlayerShoot.mp3")] private var playerShoot:Class;
		[Embed(source = "../../../media/sound/PlayerHurt.mp3")] private var playerHurt:Class;
		
		private var enemies:FlxGroup;
		private var bullets:FlxGroup;
		private var playerBullets:FlxGroup;
		private var gibs:FlxEmitter;
		
		private var movementSpeed:Number;
		private var speedReduction:Number;
		
		
		private var fireTimer:Number;
		private var fireCooldown:Number;
		
		public var attachedBullets:FlxGroup;
		
		private var invulnTime:Number;
		private var invulnCounter:Number;
		private var isInvuln:Boolean;
		
		public function Player(x:int, y:int, enemies:FlxGroup, bullets:FlxGroup, playerBullets:FlxGroup, gibs:FlxEmitter) 
		{
			super(x, y);
			this.enemies = enemies;
			this.bullets = bullets;
			this.playerBullets = playerBullets;
			this.gibs = gibs;
			attachedBullets = new FlxGroup();
			
			movementSpeed = 300;
			speedReduction = 0.9;
			
			loadGraphic(playerImage, true, false, 60, 60);
			
			addAnimation("flicker", [0, 1], 15, true);
			play("flicker");
			
			for (var i:int = 0; i < 15; i++) {
				addBullet(FlxU.random() * 360);
			}
			
			fireTimer = 1;
			fireCooldown = 0.5;
			
			isInvuln = false;
			invulnCounter = 0;
			invulnTime = 0.5;
		}
		
		private function updateMovement():void {
			var movedX:Boolean = false;
			var movedY:Boolean = false;
			
			if (FlxG.keys.LEFT || FlxG.keys.A) {
				velocity.x = -movementSpeed;
				movedX = true;
			}
			if (FlxG.keys.RIGHT || FlxG.keys.D) {
				velocity.x = movementSpeed;
				movedX = true;
			}
			if (FlxG.keys.UP || FlxG.keys.W) {
				velocity.y = -movementSpeed;
				movedY = true;
			}
			if (FlxG.keys.DOWN || FlxG.keys.S) {
				velocity.y = movementSpeed;
				movedY = true;
			}
			
			if (!movedX) {
				velocity.x *= speedReduction;
			}
			if (!movedY) {
				velocity.y *= speedReduction;
			}
		}
		
		public function addBullet(angle:Number):void {
			var bullet:PlayerBullet = new PlayerBullet(this, angle);
			playerBullets.add(bullet);
			attachedBullets.add(bullet);
		}
		
		public function onEnemyOverlap(enemy:FlxObject):void {
			addBullet(enemy.angle);
			FlxG.play(powerUpSound);
		}
		
		private function onBulletOverlap(self:FlxObject, bullet:FlxObject):void {
			if (!isInvuln) {
				bullet.kill();
				gibs.at(this);
				gibs.start(true, 0, 20);
				FlxG.play(playerHurt);
				FlxG.quake.start(0.01, 0.25);
				if (attachedBullets.members.length > 0) {
					var removedBullet:FlxObject = attachedBullets.getRandom();
					attachedBullets.remove(removedBullet, true);
					removedBullet.kill();
				} else {
					FlxG.fade.start(0xff000000, 0.5, onDeath);
				}
				isInvuln = true;
				invulnCounter = 0;
				for (var i:int = 0; i < attachedBullets.members.length; i++) {
					attachedBullets.members[i].flicker(invulnTime);
				}
			}
		}
		private function onDeath():void {
			FlxG.state = new EndState();
		}
		
		private function shootBullet():void {
			if (fireTimer >= fireCooldown) {
				fireTimer = 0;
				if (attachedBullets.members.length > 0) {
					var bullet:PlayerBullet = attachedBullets.getRandom() as PlayerBullet;
					bullet.shoot();
					attachedBullets.remove(bullet, true);
					FlxG.play(playerShoot);
				}
			}
		}
		
		private function checkCollision():void {
			FlxU.overlap(this, bullets, onBulletOverlap);
		}
		
		override public function update():void 
		{
			updateMovement();
			
			if (x < 0) {
				x = 0;
			} else if (x + width > FlxG.stage.stageWidth) {
				x = FlxG.stage.stageWidth - width;
			}
			
			if (y < 0) {
				y = 0;
			} else if (y + height > FlxG.stage.stageHeight) {
				y = FlxG.stage.stageHeight - height;
			}
			
			fireTimer += FlxG.elapsed;
			if (FlxG.keys.Z) {
				shootBullet();
			} else {
				fireTimer = fireCooldown;
			}
			
			var i:int = 0;
			if (isInvuln) {
				flicker(invulnTime);
				for (i = 0; i < attachedBullets.members.length; i++) {
					attachedBullets.members[i].flicker(invulnTime);
				}
				invulnCounter += FlxG.elapsed;
				if (invulnCounter >= invulnTime) {
					isInvuln = false;
					flicker(0.0001);
					for (i = 0; i < attachedBullets.members.length; i++) {
						attachedBullets.members[i].flicker(0.0001);
					}	
				}
			}
			
			checkCollision();
			
			super.update();
		}
	}

}